package wxapplet

import (
	"github.com/go-pay/gopay"
	"github.com/go-pay/gopay/wechat"
	"go.uber.org/zap"
	"git.getensh.com/common/gopkgs/logger"
	"google.golang.org/grpc/status"
	"fmt"
	"context"
	"property-thirdparty/errors"
	pb_v1 "property-thirdparty/pb/v1"
	"property-thirdparty/parser"
	"strconv"
	"time"
)

type WxAppletQueryParam struct {
	AppId string
	MchId string
	ApiKey string
	OrderNo string
}

func AppletQueryOrder(info *WxAppletQueryParam) (string, string, string, int64, int64, error) {
	client := wechat.NewClient(info.AppId, info.MchId, info.ApiKey, true)

	//设置国家
	client.SetCountry(wechat.China)

	bm := make(gopay.BodyMap)
	bm.Set("out_trade_no", info.OrderNo).
		Set("nonce_str", string(Krand(32, KC_RAND_KIND_ALL))).
		Set("sign_type", wechat.SignType_MD5)

	// 请求订单查询，成功后得到结果
	resp, _, err := client.QueryOrder(bm)

	if err != nil {
		param := bm.JsonBody()
		logger.Error("func",
			zap.String("call", "QueryOrder"),
			zap.String("params", param),
			zap.String("error", err.Error()))
		return "", "", "",0, 0, status.Error(10006, fmt.Sprintf("微信小程序查询失败：%s", resp.ReturnMsg))
	}
	if resp.ReturnCode == "FAIL" {
		param := bm.JsonBody()
		logger.Error("func",
			zap.String("call", "QueryOrder"),
			zap.String("params", param),
			zap.String("return_code", "FAIL"),
			zap.String("err_code", resp.ErrCode),
			zap.String("error", resp.ReturnMsg))
		return "",  "", "",0,0, status.Error(10006, fmt.Sprintf("微信小程序查询失败：%s", resp.ReturnMsg))
	}
	if resp.ResultCode == "FAIL" {
		param := bm.JsonBody()
		logger.Error("func",
			zap.String("call", "QueryOrder"),
			zap.String("params", param),
			zap.String("result_code", "FAIL"),
			zap.String("err_code", resp.ErrCode),
			zap.String("error", resp.ErrCodeDes))
		return "",   "", "",0,0, status.Error(10006, fmt.Sprintf("微信小程序查询失败：%s", resp.ErrCodeDes))
	}

	payTime := int64(0)
	amount := int64(0)
	if resp.TradeState == "SUCCESS" {
		ptime, err := time.ParseInLocation("20060102150405", resp.TimeEnd, time.Local)
		if err != nil {
			payTime = ptime.Unix()
		}
		amount, _ = strconv.ParseInt(resp.CashFee, 10, 64)

	}

	return resp.TradeState, resp.TradeStateDesc, resp.TransactionId, payTime, amount, nil
}

func WxAppletQuery(ctx context.Context, req *pb_v1.WxAppletQueryRequest) (reply *pb_v1.WxAppletQueryReply, err error) {
	reply = &pb_v1.WxAppletQueryReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	if req.Order == "" {
		return nil, errors.ParamsError
	}
	mreq := WxAppletQueryParam{
		AppId:parser.Conf.ThirdParty.Wx.AppletAppId,
		MchId:parser.Conf.ThirdParty.Wx.AppletMchId,
		ApiKey:parser.Conf.ThirdParty.Wx.AppletAppKey,
		OrderNo:req.Order,
	}


	reply.TradeState, reply.TradeStateDesc, reply.TransactionId, reply.PayTime, reply.Amount, err = AppletQueryOrder(&mreq)

	return reply, nil
}
