package wxmerchant

import (
	"context"
	"fmt"
	"github.com/tidwall/gjson"
	"github.com/wechatpay-apiv3/wechatpay-go/core"
	"github.com/wechatpay-apiv3/wechatpay-go/core/option"
	"github.com/wechatpay-apiv3/wechatpay-go/utils"
	"go.uber.org/zap"
	"io/ioutil"
	"log"
	"net/http"
	"encoding/json"
	"net/url"
	"property-thirdparty/parser"
	pb_v1 "property-thirdparty/pb/v1"
	"git.getensh.com/common/gopkgs/logger"
	"google.golang.org/grpc/status"
)


func searchApplyStatus(req *pb_v1.WxMerchantSearchRequest, reply *pb_v1.WxMerchantSearchReply) error {
	mchID := parser.Conf.ThirdParty.Wx.MerchantMchID
	mchCertificateSerialNumber := parser.Conf.ThirdParty.Wx.MerchantMchCertificateSerialNumber
	mchAPIv3Key := parser.Conf.ThirdParty.Wx.MerchantMchAPIv3Key


	// 使用 utils 提供的函数从本地文件中加载商户私钥，商户私钥会用来生成请求的签名
	mchPrivateKey, err := utils.LoadPrivateKey(parser.Conf.ThirdParty.Wx.MerchantMchPrivateKey)
	if err != nil {
		return status.Error(10002, "密钥加载失败")
	}

	ctx := context.Background()
	// 使用商户私钥等初始化 client，并使它具有自动定时获取微信支付平台证书的能力
	opts := []core.ClientOption{
		option.WithWechatPayAutoAuthCipher(mchID, mchCertificateSerialNumber, mchPrivateKey, mchAPIv3Key),
	}
	client, err := core.NewClient(ctx, opts...)
	if err != nil {
		log.Printf("new wechat pay client err:%s", err)
	}
	/*
		ctx context.Context,
			method, requestPath string,
			headerParams http.Header,
			queryParams url.Values,
			postBody interface{},
			contentType string,
	*/
	headers := http.Header{}
	headers.Set("Accept", "application/json")
	headers.Set("Content-Type", "application/json")

	rurl := "https://api.mch.weixin.qq.com/v3/applyment4sub/applyment/applyment_id/"+req.ApplymentId
	if req.BusinessCode != "" {
		rurl = "https://api.mch.weixin.qq.com/v3/applyment4sub/applyment/business_code/"+req.BusinessCode
	}
	r, err := client.Request(context.Background(), "GET", rurl, headers, url.Values{}, nil, "application/json")
	if err != nil {
		return status.Error(10003, err.Error())
	}


	contents, err := ioutil.ReadAll(r.Response.Body)
	if err != nil {
		return status.Error(10006, err.Error())
	}

	reply.ApplymentState = gjson.GetBytes(contents, "applyment_state").String()
	reply.ApplymentStateMsg = gjson.GetBytes(contents, "applyment_state_msg").String()
	reply.MchId = gjson.GetBytes(contents, "sub_mchid").String()
	reply.SignUrl = gjson.GetBytes(contents, "sign_url").String()
	reply.BusinessCode = gjson.GetBytes(contents, "business_code").String()
	reply.ApplymentId = gjson.GetBytes(contents, "applyment_id").String()
	array := gjson.GetBytes(contents, "audit_detail").Array()
	if reply.ApplymentState == "" {
		return status.Error(10006, fmt.Sprintf("未获取到状态:%s", contents))
	}
	mArray := make([]map[string]string, len(array))
	for i, v := range array {
		m := map[string]string{}
		m["field"] = v.Get("field").String()
		m["field_name"] = v.Get("field_name").String()
		m["reject_reason"] = v.Get("reject_reason").String()
		mArray[i] = m
	}
	if len(mArray) > 0 {
		bytes, _ := json.Marshal(mArray)
		reply.AuditDetail = string(bytes)
	}

	return nil

}


func WxMerchantSearch(ctx context.Context, req *pb_v1.WxMerchantSearchRequest) (reply *pb_v1.WxMerchantSearchReply, err error) {
	reply = &pb_v1.WxMerchantSearchReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	err = searchApplyStatus(req, reply)
	if err != nil {
		logger.Error("func",
			zap.String("call", "searchApplyStatus"),
			zap.String("error", err.Error()))
	}
	return reply, err
}
